/*============================================================================
      DEPTH FIRST VISIT of a directed or undirected graph from a given node.
      REACHABILITY from a given node.
The reader may want to look also at the programs for computing a spanning
forest of a (directed or undirected) graph.
------------------------------------------------------------------------------
The graph is stored as an array, say G, of lists. Each list G[i] tells us that
node i is connected to each node of the list G[i]. Consider N=9 nodes, and the
file, whose name is: graph1_in and whose content is the following sequence of
integers:
3 8 0 2 0 7 2 7 7 5 5 3 2 6 2 5 3 2 3 4 meaning that there is an arc
from node 3 to node 8, from node 0 to node 2, etc.
If the graph is undirected, 3 8 means that there is an undirected arc from 3 to
8, that is, there are two directed arcs, the first from 3 to 8 and the second
from 8 to 3. We have the following array G of lists:
[ 0   1   2   3   4   5   6   7   8]  :from_node
  7  nil  5   4  nil  3  nil  5  nil  :to_node
  |       |   |       |       |           |
  v       v   v       v       v           v
  2       6   2      nil     nil      :to_node
  |       |   |                           |
  v       v   v                           v
 nil      7   8                       :to_node
          |   |                           |
          v   v                           v
         nil nil                      :to_node
The program is correct also for directed graphs with loops.
The program uses the module for lists in the file `list_module.cpp'
=========================================================================== */
#include <fstream.h>        // for reading and writing files
typedef int Values;
#include "list_module.cpp"  // we have lists of integers
fstream input_graph;        // using a file for describing the input_graph
list reach=NULL;            // reach is a global variable keeping the output
//----------------------------------------------------------------------------
void printlistar(list A[], int size)
{cout << "["; for (int i=0; i<size; i=i+1)
     {cout << i << ":";
      printlist(A[i]); if (i!=size-1) {cout << ", ";};};
 cout << "]"; }
//----------------------------------------------------------------------------
void readgraph(list G[], int N)
{int from_node=0, to_node=0, r=0;
for (int i=0; i<N-1; i=i+1) {G[i]=NULL;};
input_graph >> from_node;        // <-- eof()=1 if we read a non existing item
while (!input_graph.eof())
     {input_graph >> to_node;
/*----------------------------------------------------------------------------
We consider a directed graph: we now insert the arc <from_node, to_node>.
--------------------------------------------------------------------------- */
      G[from_node]=cons(to_node,G[from_node]);
/*----------------------------------------------------------------------------
For undirected graphs we should insert also the arc <to_node, from_node>
using the statement: G[to_node]=cons(from_node,G[to_node]);
--------------------------------------------------------------------------- */
      input_graph >> from_node;      // <-- trying to read the first node
      };}                            //     of a new <from_node, to_node> pair
//----------------------------------------------------------------------------
void reachable(int n, list G[])
{if (!member(n,reach))
  {reach=cons(n,reach); list edge=G[n];
  while (edge!=NULL)
      {reachable(head(edge),G); edge=tail(edge);};};}   // depth_first_visit
//----------------------------------------------------------------------------
void main()
{const int N=9;
int initialnode; list Graph [N]={};
input_graph.open("graph1_in", ios::in); readgraph(Graph,N);
cout << "The given directed graph represented as array of lists, is:\n";
printlistar(Graph,N); cout << "\n\n";
cout << "The initial node is:\n" <<
        "(type an integer from 0 to " << N-1 <<" and CR)\n";
cin >> initialnode; reachable(initialnode,Graph);
cout << "Set of nodes reachable from node " << initialnode << ":\n";
printlistasset(reach); cout << "\n";
}
/* input:      output:
------------------------------------------------------------------------------
               The given directed graph represented as array of lists, is:
               [0:7->2->nil, 1:nil, 2:5->6->7->nil, 3:4->2->8->nil, 4:nil,
                5:3->nil, 6:nil, 7:5->nil, 8:nil]

               The initial node is:
               (type an integer from 0 to 8 and CR)
 3  CR
               Set of nodes reachable from node 3:
               {8, 7, 6, 5, 2, 4, 3}
--------------------------------------------------------------------------- */